//1 Manejo de sesin
//2 Manejo del orden de invocacin
//3 Secuencia de los mensajes y retries
//4 Record de mensajes por msmq (mostrar como se procesan desconectados)
//5 Transacionalidad

using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Transactions;

namespace Client
{
    public class Program
    {
        static void Main( string[ ] args )
        {
            Console.WriteLine( "Press any key when the service is ready." );
            Console.ReadKey( );

            do
            {
                Console.WriteLine( );

                using ( TransactionScope scope = new TransactionScope( TransactionScopeOption.RequiresNew ) )
                {
                    decimal? dealValue = null;
                    decimal? hedgeValue = null;

                    ITradingService dealProxy = null;
                    ITradingService hedgeProxy = null;
                    try
                    {
                        //1, 2
                        dealProxy = new ChannelFactory<ITradingService>( "TradingServiceConfiguration" ).CreateChannel( );

                        //3
                        //dealProxy = new ChannelFactory<ITradingService>( new ReliableHttpBinding( ), new EndpointAddress( "http://localhost:8001/TradingService" ) ).CreateChannel( );

                        //1, 2
                        dealProxy.BeginDeal( );

                        Trade trade = new Trade( );
                        trade.Count = 10;
                        trade.Symbol = "MSFT";
                        trade.Date = DateTime.Now.AddMonths( 2 );
                        dealProxy.AddTrade( trade );

                        dealProxy.AddFunction( "InterestRateEstimation" );
                        dealProxy.AddFunction( "TechnologyStockEstimation" );

                        dealValue = dealProxy.Calculate( );

                        dealProxy.Purchase( );

                        dealProxy.EndDeal( );

                        hedgeProxy = new ChannelFactory<ITradingService>( "TradingServiceConfiguration" ).CreateChannel( );

                        hedgeProxy.BeginDeal( );

                        trade = new Trade( );
                        trade.Count = 10;
                        trade.Symbol = "MSFT";
                        trade.Date = DateTime.Now.AddMonths( 2 );
                        hedgeProxy.AddTrade( trade );

                        hedgeProxy.AddFunction( "InterestRateEstimation" );
                        hedgeProxy.AddFunction( "TechnologyStockEstimation" );

                        hedgeValue = hedgeProxy.Calculate( );

                        hedgeProxy.Purchase( );

                        hedgeProxy.EndDeal( );
                                                  
                        //5
                        //if ( ( dealValue != null ) && ( hedgeValue != null ) )
                        //{
                        //    Console.WriteLine( "Deal value is ${0}, and hedge value is ${1}", dealValue.Value, hedgeValue.Value );
                        //    if ( ( dealValue.Value - hedgeValue.Value ) > 20m )
                        //    {
                        //        Console.WriteLine( "Voting to complete trade!" );
                        //        scope.Complete( );
                        //    }
                        //    else
                        //    {
                        //        Console.WriteLine( "Voting NOT to complete trade!  Transaction will not complete. Keep trying until a transaction commits." );
                        //    }
                        //}
                        scope.Complete( );
                    }
                    catch ( Exception exception )
                    {
                        Console.WriteLine( exception.Message );
                    }
                    finally
                    {
                        if ( dealProxy != null )
                            ( ( IChannel )dealProxy ).Close( );
                        if ( hedgeProxy != null )
                            ( ( IChannel )hedgeProxy ).Close( );
                    }
                }

                Console.WriteLine( "Press q to quit or any other key to continue." );
            }
            while ( Console.ReadKey( ).KeyChar != 'q' );

            Console.WriteLine( );
            Console.WriteLine( "Done." );
            Console.ReadKey( );
        }
    }
}