using System;
using System.Collections.Generic;
using System.ServiceModel;

namespace Fabrikam
{
    [ServiceBehavior( InstanceContextMode = InstanceContextMode.PerSession )]
    public class TradingSystem : ITradingService
    {
        #region ITradingService Members

        string ITradingService.BeginDeal( )
        {
            string dealIdentifier = Guid.NewGuid( ).ToString( );

            OperationContext.Current.InstanceContext.Extensions.Add( new DealData( dealIdentifier ) );

            Console.WriteLine( "Started deal {0}", dealIdentifier );
            return dealIdentifier;
        }

        void ITradingService.AddTrade( Trade trade )
        {
            DealData dealData = OperationContext.Current.InstanceContext.Extensions.Find<DealData>( );
            dealData.AddTrade( trade );
            Console.WriteLine( "Added trade for {0}", trade );
        }

        void ITradingService.AddFunction( string function )
        {
            Console.WriteLine( "Added function {0}", function );
        }

        decimal ITradingService.Calculate( )
        {
            Decimal value = DateTime.Now.Millisecond / 10;
            Console.WriteLine( "Calculated value as {0}", value );
            return value;
        }

        [OperationBehavior( TransactionScopeRequired = true, TransactionAutoComplete = true )]
        void ITradingService.Purchase( )
        {
            DealData dealData = OperationContext.Current.InstanceContext.Extensions.Find<DealData>( );

            //4
            ITradeRecorder proxy = new ChannelFactory<ITradeRecorder>( "TradeRecordingService" ).CreateChannel( );
            proxy.RecordTrades( dealData.Trades );
            
            foreach ( Trade trade in dealData.Trades )
            {
                Console.WriteLine( string.Format( "Purchased {0}", trade ) );
            }
        }

        void ITradingService.EndDeal( )
        {
            DealData dealData = OperationContext.Current.InstanceContext.Extensions.Find<DealData>( );
            Console.WriteLine( "Completed deal: {0}", dealData.DealIdentifier );
        }

        #endregion
    }

    internal class DealData : IExtension<InstanceContext>
    {
        private string dealIdentifier = null;
        private List<Trade> trades = null;

        public DealData( string dealIdentifier )
        {
            this.dealIdentifier = dealIdentifier;
            this.trades = new List<Trade>( );
        }

        public string DealIdentifier
        {
            get
            {
                return this.dealIdentifier;
            }
        }

        public void AddTrade( Trade trade )
        {
            trades.Add( trade );
        }

        public Trade[ ] Trades
        {
            get
            {
                return this.trades.ToArray( );
            }
        }

        #region IExtension<InstanceContext> Members

        void IExtension<InstanceContext>.Attach( InstanceContext owner ) { }

        void IExtension<InstanceContext>.Detach( InstanceContext owner ) { }

        #endregion
    }
}