using System;
using System.Collections.ObjectModel;
using System.Configuration;
using System.IdentityModel.Claims;
using System.IdentityModel.Tokens;
using System.ServiceModel;
using System.Web.Security;
using SecurityTokenService.DataContracts;
using SecurityTokenService.Configuration;
using SecurityTokenService.Utility;

namespace SecurityTokenService
{
    public sealed class STSUsingASPNETRoleProviderImplementation : SecurityTokenService
    {
        /// <summary>
        /// Default constructor 
        /// </summary>
        public STSUsingASPNETRoleProviderImplementation()
            : base(
                Constants.StsName,
                FederationUtilities.GetX509TokenFromCert(ServiceConfiguration.Security.IssuerCertificate.StoreName,
                                                         ServiceConfiguration.Security.IssuerCertificate.StoreLocation,
                                                         ServiceConfiguration.Security.IssuerCertificate.
                                                             CertDistinguishedName),
                FederationUtilities.GetX509TokenFromCert(ServiceConfiguration.Security.TargetCertificate.StoreName,
                                                         ServiceConfiguration.Security.TargetCertificate.StoreLocation,
                                                         ServiceConfiguration.Security.TargetCertificate.
                                                             CertDistinguishedName))
        {
        }

        /// <summary>
        /// Service configuration section elements
        /// </summary>
        public static STSConfigurationSection ServiceConfiguration
        {
            get
            {
                return
                    (STSConfigurationSection)
                    ConfigurationManager.GetSection(Constants.ApplicationConfigurationSectionName);
            }
        }

        /// <summary>
        /// Gets the subject name used to create the SAML token.
        /// </summary>
        /// <returns>Subject name</returns>
        protected override string GetSubjectName()
        {
            return ServiceSecurityContext.Current.PrimaryIdentity.Name;
        }

        /// <summary>
        /// Gets claims to be added to the SAML Token issued by the STS
        /// </summary>
        /// <param name="requestSecurityToken">Request message sent by clients</param>
        /// <returns>Collection of SAML claims to be added to the response token</returns>
        protected override Collection<SamlAttribute> GetIssuedClaims(RequestSecurityToken requestSecurityToken)
        {
            string callerIdentityName = GetSubjectName();

            Console.WriteLine(
                "\nIssuing SAML Token for {0} with the following roles:", 
                callerIdentityName);
            
            //
            // Create Name Claim
            //
            Collection<SamlAttribute> samlAttributes = new Collection<SamlAttribute>();
            samlAttributes.Add(new SamlAttribute(Claim.CreateNameClaim(callerIdentityName)));

            //
            // Create Role Claim Collection using configured ASP.NET RoleProvider
            //
            foreach(string role in Roles.GetRolesForUser(callerIdentityName))
            {
                samlAttributes.Add(
                    new SamlAttribute(new Claim(Constants.RoleClaim, role, Rights.PossessProperty)));

                Console.WriteLine("\tRole = {0}...", role);
            }

            return samlAttributes;
        }
    }
}